// Protokoll fuer die Datenuebertragung zum Sensor

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

#define DDR_SPI DDRB			// Data Direction Register for SPI
#define PORT_SPI PORTB			// PORT B is used for SPI Interface
#define reset DDB3				// Pin 4 = Reset
#define SS DDB4					// Pin 5 = Slave Select
#define MOSI DDB5				// Pin 6 = Master Out Slave In
#define MISO DDB6				// Pin 7 = Master In Slave Out
#define SCK DDB7				// Pin 8 = Clock


void SPI_init(void)											// Initialisiert SPI
{	
	DDR_SPI=(1<<MOSI)|(1<<SCK)|(1<<SS)|(1<<reset);		// SCK, SS, MOSI and Reset output, others input
	DDR_SPI&=~(1<<MISO);									// MISO Input
	

	
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<CPOL);
	// SPI Enable, SPI as Master, F_OSC/64, Clock Polarity
	SPSR=(1<<SPI2X);										// double SPI Frequency
	// Clock-Rate: 500kHz

	PORT_SPI|=(1<<reset);									// Toggle Reset 250s
	_delay_us(250);
	PORT_SPI&=~(1<<reset);
	PORT_SPI|=(1<<SS);										// Slave Select is high - only low when transmission
}
	

void SPI_transmit(uint8_t data, uint8_t eot)				// Transmit 1 Byte
{
	PORT_SPI&=~(1<<SS);									// SS low - transmission follows
	// _delay_us(5);
	
	SPDR=data;												// Data to SPI-Data-Register
	while(!(SPSR&(1<<SPIF)))	{}							// Wait for end of transmission
	
	if(eot==1) PORT_SPI|=(1<<SS);							// Letztes Byte gesendet - SS high schliet Uebertragung ab
}

uint8_t SPI_receive(uint8_t eot)									// Receive 1 Byte
{
	PORT_SPI&=~(1<<SS);									// SS low - transmission follows
	
	// SPDR=0xFF;												// Writing initiates data transmission
	while(!(SPSR&(1<<SPIF)))	{}							// wait for reception complete
	
	if(eot==1) PORT_SPI|=(1<<SS);							// Letztes Byte uebertragen
	
	return SPDR;											// returns received data
}
